@@ -14,6 +14,69 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+#' create a hypothesis object
+#' @param xs a numeric vector
+#' @param type a string
+#' @return a list containing the following elements
+#' @examples 
+#' h1 <- hypothesis(c(2, 1, 3, 4), type = "pairwise")
+#' h2 <- hypothesis(c(2, 1, 3, 4), type = "adjacent")
+#' @export
+hypothesis <- function(xs, type = "pairwise") {
+
+  stopifnot("A vector or numeric values in required"=is.numeric(xs))
+  stopifnot("A vector containing at least two elements is required"=length(xs) > 1)
+  stopifnot("type must be either 'pairwise' or 'adjacent'"=type %in% c("pairwise", "adjacent"))
+
+  xs_length <- length(xs)
+
+  if (type == "pairwise") {
+    n_pairs <- ((xs_length - 1) * xs_length) / 2
+  } else if (type == "adjacent") {
+    n_pairs <- xs_length - 1
+  }
+
+  ordinal_relations <- ordering(xs, type, 0)
+  n_increases <- length(ordinal_relations[ordinal_relations == 1])
+  n_decreases <- length(ordinal_relations[ordinal_relations == -1])
+  n_equalities <- length(ordinal_relations[ordinal_relations == 0])
+
+  structure(
+    list(
+      raw = xs,
+      type = type,
+      n_pairs = n_pairs,
+      ordinal_relations = ordinal_relations,
+      n_increases = n_increases,
+      n_decreases = n_decreases,
+      n_equalities = n_equalities
+    ),
+    class = "opahypothesis"
+  )
+}
+
+#' Print details of a hypothesis
+#' @param x an object of type "opaHypothesis"
+#' @param ... ignored
+#' @return No return value, called for side-effects.
+#' @examples 
+#' h1 <- hypothesis(c(2, 1, 3, 4), type = "pairwise")
+#' print(h1)
+#' h2 <- hypothesis(c(2, 1, 3, 4), type = "adjacent")
+#' print(h2)
+#' @export 
+print.opahypothesis <- function(x, ...) {
+  cat("********** Ordinal Hypothesis **********\n")
+  cat("Hypothesis type:", x$type, "\n")
+  cat("Raw hypothesis:\n", x$raw, "\n")
+  cat("Ordinal relations:\n", x$ordinal_relations, "\n")
+  cat("N conditions:", length(x$raw), "\n")
+  cat("N hypothesised ordinal relations:", x$n_pairs, "\n")
+  cat("N hypothesised increases:", x$n_increases, "\n")
+  cat("N hypothesised decreases:", x$n_decreases, "\n")
+  cat("N hypothesised equalities:", x$n_equalities, "\n")
+}
+
 
 #' Prints a summary of results from a fitted ordinal pattern analysis model.
 #' @param object an object of class "opafit".
@@ -24,7 +87,8 @@
 #' dat <- data.frame(t1 = c(9, 4, 8, 10),
 #'                   t2 = c(8, 8, 12, 10),
 #'                   t3 = c(8, 5, 10, 11))
-#' opamod <- opa(dat, 1:3)
+#' h <- hypothesis(1:3)
+#' opamod <- opa(dat, h)
 #' summary(opamod)
 #' summary(opamod, digits = 3)
 #' @export
@@ -54,7 +118,8 @@ summary.opafit <- function(object, ..., digits = 2L) {
 #' dat <- data.frame(t1 = c(9, 4, 8, 10),
 #'                   t2 = c(8, 8, 12, 10),
 #'                   t3 = c(8, 5, 10, 11))
-#' opamod <- opa(dat, 1:3)
+#' h <- hypothesis(1:3)
+#' opamod <- opa(dat, h)
 #' pw <- compare_conditions(opamod)
 #' print(pw)
 #' print(pw, digits = 2)
@@ -90,7 +155,8 @@ print.pairwiseopafit <- function(x, ...) {
 #' dat <- data.frame(t1 = c(9, 4, 8, 10),
 #'                   t2 = c(8, 8, 12, 10),
 #'                   t3 = c(8, 5, 10, 11))
-#' opamod <- opa(dat, 1:3)
+#' h <- hypothesis(1:3)
+#' opamod <- opa(dat, h)
 #' print(opamod)
 #' @export
 print.opafit <- function(x, ...) {
@@ -102,17 +168,28 @@ print.opafit <- function(x, ...) {
 #' @param m an object of class "opafit"
 #' @param threshold a boolean indicating whether to plot a threshold abline
 #' @param title a boolean indicating whether to include a plot title
-#' @param legend a boolean indicating whether to include a legend when n groups > 1
+#' @param legend a boolean indicating whether to include a legend
 #' @return No return value, called for side effects.
 #' @examples
 #' dat <- data.frame(t1 = c(9, 4, 8, 10),
 #'                   t2 = c(8, 8, 12, 10),
 #'                   t3 = c(8, 5, 10, 11))
-#' opamod <- opa(dat, 1:3)
+#' h <- hypothesis(1:3)
+#' opamod <- opa(dat, h)
 #' pcc_plot(opamod)
 #' pcc_plot(opamod, threshold = 85)
 #' @export
 pcc_plot <- function(m, threshold = NULL, title = TRUE, legend = TRUE) {
+  UseMethod("pcc_plot")
+}
+
+
+#' @export
+pcc_plot.default <- function(m, threshold = NULL, title = TRUE, legend = TRUE) .NotYetImplemented()
+
+
+#' @export
+pcc_plot.opafit <- function(m, threshold = NULL, title = TRUE, legend = TRUE) {
   old_par <- par()
   if (is.null(m$groups)) {
     par(mar = c(4, 4, 2, 1)) # no legend for single group
@@ -163,7 +240,7 @@ pcc_plot <- function(m, threshold = NULL, title = TRUE, legend = TRUE) {
   if (!is.null(m$groups)) {
     if (legend == TRUE) {
       legend("right", legend = levels(m$groups), title = "Group",
-             pch=21, pt.bg = m$groups, cex=1,
+             pch=21, pt.bg = 1:nlevels(m$groups), cex=1,
              xpd = TRUE, inset = c(-0.3, 0))
     }
   }
@@ -181,11 +258,22 @@ pcc_plot <- function(m, threshold = NULL, title = TRUE, legend = TRUE) {
 #' dat <- data.frame(t1 = c(9, 4, 8, 10),
 #'                   t2 = c(8, 8, 12, 10),
 #'                   t3 = c(8, 5, 10, 11))
-#' opamod <- opa(dat, 1:3)
+#' h <- hypothesis(1:3)
+#' opamod <- opa(dat, h)
 #' cval_plot(opamod)
 #' cval_plot(opamod, threshold = 0.1)
 #' @export
 cval_plot <- function(m, threshold = NULL, title = TRUE, legend = TRUE) {
+  UseMethod("cval_plot")
+}
+
+
+#' @export
+cval_plot.default <- function(m, threshold = NULL, title = TRUE, legend = TRUE) .NotYetImplemented()
+
+
+#' @export
+cval_plot.opafit <- function(m, threshold = NULL, title = TRUE, legend = TRUE) {
   old_par <- par()
   if (is.null(m$groups)) {
     par(mar = c(4, 4, 2, 1))
@@ -237,7 +325,7 @@ cval_plot <- function(m, threshold = NULL, title = TRUE, legend = TRUE) {
   if (!is.null(m$groups)) {
     if (legend == TRUE) {
       legend("right", legend = levels(m$groups), title = "Group",
-             pch=21, pt.bg = m$groups, cex=1,
+             pch=21, pt.bg = 1:nlevels(m$groups), cex=1,
              xpd = TRUE, inset = c(-0.3, 0))
     }
   }
@@ -255,15 +343,16 @@ cval_plot <- function(m, threshold = NULL, title = TRUE, legend = TRUE) {
 #' dat <- data.frame(t1 = c(9, 4, 8, 10),
 #'                   t2 = c(8, 8, 12, 10),
 #'                   t3 = c(8, 5, 10, 11))
-#' opamod <- opa(dat, 1:3)
+#' h <- hypothesis(1:3)
+#' opamod <- opa(dat, h)
 #' plot(opamod)
 #' @export
 plot.opafit <- function(x, pcc_threshold = NULL, cval_threshold = NULL, ...) {
   old_par <- par()
   if (is.null(x$groups)) {
     par(mfrow = c(1, 2))
-    pcc_plot(x, threshold = pcc_threshold)
-    cval_plot(x, threshold = cval_threshold)
+    pcc_plot(x, threshold = pcc_threshold, legend = FALSE)
+    cval_plot(x, threshold = cval_threshold, legend = FALSE)
     par(mfrow = c(1, 1))
   } else {
     layout(matrix(c(1, 2, 3, 3), ncol = 2, byrow = TRUE), heights=c(4, 1))
@@ -272,14 +361,35 @@ plot.opafit <- function(x, pcc_threshold = NULL, cval_threshold = NULL, ...) {
     cval_plot(x, threshold = cval_threshold, legend = FALSE)
     par(mai = c(0, 0, 0, 0))
     plot.new()
-    legend(x="center", horiz = TRUE, legend = levels(x$groups), title = "Group",
-           pch=21, pt.bg = x$groups, cex=1)
+    legend(x="center", horiz = TRUE, legend = levels(x$groups), title = "Group", pch=21, pt.bg = 1:nlevels(x$groups), cex=1)
     par(mfrow = c(1, 1))
   }
   par(mar = old_par$mar)
 }
 
 
+#' Plot a hypothesis.
+#' @param x an object of class "opaHypothesis"
+#' @param title a boolean indicating whether to include a plot title
+#' @param ... ignored
+#' @return No return value, called for side effects.
+#' @examples
+#' h <- hypothesis(c(1,2,3,3,3))
+#' plot(h)
+#' @export
+plot.opahypothesis <- function(x, title = TRUE, ...) {
+  par(mar = c(4, 4, 2, 0.5))
+  plot(x = NULL, y = NULL, xlim = c(0.5, length(x$raw) + 0.5),
+       ylim = c(min(x$raw) - 0.5, max(x$raw) + 0.5),
+       xlab = "x", ylab = "h(x)",
+       yaxt = "n", xaxt="n",
+       main = ifelse(title == TRUE, "Hypothesis", ""))
+  points(1:(length(x$raw)), x$raw, pch=21, cex=2, bg = palette()[1])
+  axis(1, at=1:length(x$raw), labels=1:length(x$raw))
+  axis(2, at=c(min(x$raw), max(x$raw)), labels = c("Lower", "Higher"), las = 1)
+}
+
+
 #' Group-level PCC and chance values.
 #'
 #' @details
@@ -294,7 +404,8 @@ plot.opafit <- function(x, pcc_threshold = NULL, cval_threshold = NULL, ...) {
 #' dat <- data.frame(t1 = c(9, 4, 8, 10),
 #'                   t2 = c(8, 8, 12, 10),
 #'                   t3 = c(8, 5, 10, 11))
-#' opamod <- opa(dat, 1:3)
+#' h <- hypothesis(1:3)
+#' opamod <- opa(dat, h)
 #' group_results(opamod)
 #' @export
 group_results <- function(m, digits) {
@@ -327,9 +438,9 @@ group_results.opafit <- function(m, digits = 2) {
 #' Individual-level PCC and chance values.
 #'
 #' @details
-#' If the model was fitted with no grouping variable, a matrix of PCCs and
-#' c-values are returned corresponding to the order of rows in the data. If the
-#' \code{opa} model was fitted with a grouping variable specified, a table of
+#' If the \code{opa} model was fitted with no grouping variable, a matrix of PCCs
+#' and c-values are returned corresponding to the order of rows in the data. If
+#' the \code{opa} model was fitted with a grouping variable specified, a table of
 #' PCCs and c-values is returned ordered by factor level of the grouping
 #' variable.
 #' @param m an object of class "opafit" produced by \code{opa()}
@@ -340,7 +451,8 @@ group_results.opafit <- function(m, digits = 2) {
 #' dat <- data.frame(t1 = c(9, 4, 8, 10),
 #'                   t2 = c(8, 8, 12, 10),
 #'                   t3 = c(8, 5, 10, 11))
-#' opamod <- opa(dat, 1:3)
+#' h <- hypothesis(1:3)
+#' opamod <- opa(dat, h)
 #' individual_results(opamod)
 #' @export
 individual_results <- function(m, digits) {
@@ -357,7 +469,7 @@ individual_results.opafit <- function(m, digits = 2) {
   if (is.null(m$groups)) {
     out <- round(cbind(m$individual_pccs, m$individual_cvals), digits)
     colnames(out) <- c("PCC", "cval")
-    rownames(out) <- seq(length(m$individual_pccs))
+    rownames(out) <- 1:(length(m$individual_pccs))
     return(out)
   } else {
     out <- round(cbind(m$individual_idx, m$individual_pccs, m$individual_cvals), digits)
@@ -367,25 +479,197 @@ individual_results.opafit <- function(m, digits = 2) {
   }
 }
 
-
-#' Plot a hypothesis.
-#' @param h a numeric vector
-#' @param title a boolean indicating whether to include a plot title
-#' @return No return value, called for side effects.
+#' Return the group PCCs of the specified model
+#' @param m an object of class "opafit"
+#' @return a numeric vector
 #' @examples
-#' h <- c(1,2,3,3,3)
-#' plot_hypothesis(h)
+#' dat <- data.frame(t1 = c(9, 4, 8, 10),
+#'                   t2 = c(8, 8, 12, 10),
+#'                   t3 = c(8, 5, 10, 11))
+#' h <- hypothesis(1:3)
+#' opamod <- opa(dat, h)
+#' group_pccs(opamod)
 #' @export
-plot_hypothesis <- function(h, title = TRUE) {
-  par(mar = c(4, 4, 2, 0.5))
-  plot(x = NULL, y = NULL, xlim = c(0.5, length(h) + 0.5),
-       ylim = c(min(h) - 0.5, max(h) + 0.5),
-       xlab = "x", ylab = "h(x)",
-       yaxt = "n", xaxt="n",
-       main = ifelse(title == TRUE, "Hypothesis", ""))
-  points(seq(length(h)), h, pch=21, cex=2, bg = palette()[1])
-  axis(1, at=h, labels=h)
-  axis(2, at=c(min(h), max(h)), labels = c("Lower", "Higher"), las = 1)
+group_pccs <- function(m) {
+  UseMethod("group_pccs")
+}
+
+
+#' @export
+group_pccs.default <- function(m) .NotYetImplemented()
+
+
+#' @export
+group_pccs.opafit <- function(m) {
+  m$group_pcc
+}
+
+
+#' @export
+group_pccs.pairwiseopafit <- function(m) {
+  m$pccs
+}
+
+
+#' Return the group chance values of the specified model
+#' @param m an object of class "opafit"
+#' @return a numeric vector
+#' @examples
+#' dat <- data.frame(t1 = c(9, 4, 8, 10),
+#'                   t2 = c(8, 8, 12, 10),
+#'                   t3 = c(8, 5, 10, 11))
+#' h <- hypothesis(1:3)
+#' opamod <- opa(dat, h)
+#' group_cvals(opamod)
+#' @export
+group_cvals <- function(m) {
+  UseMethod("group_cvals")
+}
+
+
+#' @export
+group_cvals.default <- function(m) .NotYetImplemented()
+
+
+#' @export
+group_cvals.opafit <- function(m) {
+  m$group_cval
+}
+
+
+#' @export
+group_cvals.pairwiseopafit <- function(m) {
+  m$cvals
+}
+
+
+#' Return the individual PCCs of the specified model
+#' @param m an object of class "opafit"
+#' @return a numeric vector
+#' @examples
+#' dat <- data.frame(t1 = c(9, 4, 8, 10),
+#'                   t2 = c(8, 8, 12, 10),
+#'                   t3 = c(8, 5, 10, 11))
+#' h <- hypothesis(1:3)
+#' opamod <- opa(dat, h)
+#' individual_pccs(opamod)
+#' @export
+individual_pccs <- function(m) {
+  UseMethod("individual_pccs")
+}
+
+
+#' @export
+individual_pccs.default <- function(m) .NotYetImplemented()
+
+
+#' @export
+individual_pccs.opafit <- function(m) {
+  c(m$individual_pccs)
+}
+
+
+#' Return the individual chance values of the specified model
+#' @param m an object of class "opafit"
+#' @return a numeric vector
+#' @examples
+#' dat <- data.frame(t1 = c(9, 4, 8, 10),
+#'                   t2 = c(8, 8, 12, 10),
+#'                   t3 = c(8, 5, 10, 11))
+#' h <- hypothesis(1:3)
+#' opamod <- opa(dat, h)
+#' individual_cvals(opamod)
+#' @export
+individual_cvals <- function(m) {
+  UseMethod("individual_cvals")
+}
+
+
+#' @export
+individual_cvals.default <- function(m) .NotYetImplemented()
+
+
+#' @export
+individual_cvals.opafit <- function(m) {
+  c(m$individual_cvals)
+}
+
+
+#' Return the random order generated PCCs used to calculate the group chance value
+#' @param m an object of class "opafit"
+#' @return a numeric vector
+#' @examples
+#' dat <- data.frame(t1 = c(9, 4, 8, 10),
+#'                   t2 = c(8, 8, 12, 10),
+#'                   t3 = c(8, 5, 10, 11))
+#' h <- hypothesis(1:3)
+#' opamod <- opa(dat, h)
+#' random_pccs(opamod)
+#' @export
+random_pccs <- function(m) {
+  UseMethod("random_pccs")
+}
+
+
+#' @export
+random_pccs.default <- function(m) .NotYetImplemented()
+
+
+#' @export
+random_pccs.opafit <- function(m) {
+  m$rand_pccs
+}
+
+
+#' Return the number of pairs of observations matched by the hypothesis
+#' @param m an object of class "opafit"
+#' @return a non-negative integer
+#' @examples
+#' dat <- data.frame(t1 = c(9, 4, 8, 10),
+#'                   t2 = c(8, 8, 12, 10),
+#'                   t3 = c(8, 5, 10, 11))
+#' h <- hypothesis(1:3)
+#' opamod <- opa(dat, h)
+#' correct_pairs(opamod)
+#' @export
+correct_pairs <- function(m) {
+  UseMethod("correct_pairs")
+}
+
+
+#' @export
+correct_pairs.default <- function(m) .NotYetImplemented()
+
+
+#' @export
+correct_pairs.opafit <- function(m) {
+  m$correct_pairs
+}
+
+
+#' Return the number of pairs of observations not matched by the hypothesis
+#' @param m an object of class "opafit"
+#' @return a non-negative integer
+#' @examples
+#' dat <- data.frame(t1 = c(9, 4, 8, 10),
+#'                   t2 = c(8, 8, 12, 10),
+#'                   t3 = c(8, 5, 10, 11))
+#' h <- hypothesis(1:3)
+#' opamod <- opa(dat, h)
+#' incorrect_pairs(opamod)
+#' @export
+incorrect_pairs <- function(m) {
+  UseMethod("incorrect_pairs")
+}
+
+
+#' @export
+incorrect_pairs.default <- function(m) .NotYetImplemented()
+
+
+#' @export
+incorrect_pairs.opafit <- function(m) {
+  m$total_pairs - m$correct_pairs
 }
 
 
